jQuery( document ).ready( function( $ ) {

    /** RIDDLE LIST */

    $(".btn-riddle-clipboard").on("click", function() {
        var riddleId = $(this).attr("data-riddle-id");
        if( riddleId == undefined )
            return;

        var shortcode = "[rid=" + riddleId + "]"; // Generate Shortcode

        var textField = $(".magic-clipboard");
        textField.parent().show(); // The text field is hidden by default

        textField.val( shortcode ).select(); // select the text
        document.execCommand("copy"); // Copy; unfortunately this doesn't work with every browser
        textField.blur(); // Unfocus

        $(this).parent().find('.alert').fadeIn(1000).fadeOut(500);
    });

    $(".riddle-select").on("change", function() {
        $(".team-select-form").submit();
    });

    // prevent default when the user presses the enter key
    $(document).keypress(
        function(event){
            if (event.which == '13') {
            event.preventDefault();
        }
    });

    /** LEAD FIELD MANAGEMENT V2 */

    var dragging = Date.now();
    $( "#sortable" ).sortable();
    $( "#sortable" ).disableSelection();
    getLeadfieldsFromChildren(); // update order

    $("#sortable").on( "sortstart", function(event) {
        dragging = 0;
    });

    $("#sortable").on( "sortstop", function(event) {
        dragging = Date.now();

        getLeadfieldsFromChildren(); // update order
        $('.crp-form-input').trigger('input');
    });

    $('.btn-leadfield').on('click', function() {
        // add a 50ms cooldown so the items don't get "clicked" after stopping to drag them
        if (dragging == 0 || Date.now() - dragging <= 50) {
            return;
        }

        if ($(this).hasClass('label-input')) {
            return;
        }

        var leadField = $(this).parent();

        if (leadField.hasClass('btn-primary')) { // is selected/active
            leadField.removeClass('btn-primary');
            leadField.addClass('btn-secondary');
        } else {
            leadField.removeClass('btn-secondary');
            leadField.addClass('btn-primary');
        }

        getLeadfieldsFromChildren(); // update
        loadCRPPreview();
    });

    $('.label-input').on('input', function() {
        getLeadfieldsFromChildren(); // update labels
        loadCRPPreview();
    });

    function getLeadfieldsFromChildren()
    {
        var fields = [];
        var fieldsOrder = [];
        var fieldLabels = {};

        $('#sortable').children().each(function(index, item) {
            var fieldName = item.firstElementChild.attributes.field.value;
            var label = item.children[1].value;
            fieldsOrder.push(fieldName);

            if ('' !== label) { // is not an empty label
                fieldLabels[fieldName] = item.children[1].value;
            }

            if (item.className.includes('btn-primary')) { // is selected
                fields.push(fieldName);
                $('.icon-'+fieldName).removeClass('dashicons-plus');
                $('.icon-'+fieldName).addClass('dashicons-minus');
            } else {
                $('.icon-'+fieldName).addClass('dashicons-plus');
                $('.icon-'+fieldName).removeClass('dashicons-minus');
            }
        });

        $('#leadfieldNames').val(fields.join(','));
        $('#leadfieldNamesOrder').val(fieldsOrder.join(','));
        $('#leadfieldLabels').val(JSON.stringify(fieldLabels));

        return fields;
    }

    /* "HOW MANY ENTRIES SHOULD BE DISPLAYED" */

    $('#displayModeSelect').on('change', function() {
        var val = $(this).val();
        var input = $('.displayModeInput');
        var inputDiv = $('.displayModeInputDiv');

        if ('-1' == val || '10' == val) {
            inputDiv.hide();
        } else {
            inputDiv.show();
        }

        input.val(val);
    });

    /** LANDINGPAGE PREVIEW */

    $('.crp-form-input').on('input', function() {
        loadCRPPreview();
    });

    function loadCRPPreview()
    {
        var formData = $('.crp-form').serialize();
        var glueChar = previewUrl.includes('?') ? '&' : '?'; // example.com?test=asd&data=... VS. example.com?data=... (choose between '?' and '&')
        var url = previewUrl + glueChar + formData;

        $('#crp-preview').load(url);
    }

    // load preview on start if the variable is defined (= user is on an edit leaderboard page)
    if (typeof renderPreviewOnStart !== 'undefined' && renderPreviewOnStart) {
        loadCRPPreview();
    }

    // mediaUploader

    var mediaUploader;
    $('#riddlePluginLeaderboardUploadPicture').click(function(e) {
        e.preventDefault();
        if (mediaUploader) {
            mediaUploader.open();

            return;
        }
        mediaUploader = wp.media.frames.file_frame = wp.media(
            {
                title: 'Choose Image',
                button: {
                text: 'Choose Image'
            }, 
            multiple: false 
        });
        mediaUploader.on('select', function() {
            var attachment = mediaUploader.state().get('selection').first().toJSON();
            $('#riddleLeaderboardPictureURL').val(attachment.url);
            $('#riddleLeaderboardPictureAlt').val(attachment.alt);
            loadCRPPreview(); // reload preview
        });
        mediaUploader.open();
    });

    // remove image button
    $('#btn-remove-img').on('click', function() {
        $('#riddleLeaderboardPictureURL').val('null');
        $('#riddleLeaderboardPictureAlt').val('');
        loadCRPPreview();
    });
});