import fetchJsonp from './node_modules/fetch-jsonp/build/fetch-jsonp';

const { __, setLocaleData } = wp.i18n;
const {
	registerBlockType,
} = wp.blocks;
const {
	PlainText
} = wp.editor;
const { Button } = wp.components;

registerBlockType( 'riddle-plugin/riddle-gutenberg-block', {
	title: __( 'Riddle Quiz Maker Embed', 'riddle-plugin' ),
	icon: { 
		src: <svg viewBox="0 0 36 24">
			<g fill="#13136C" fill-rule="evenodd">

			<path d="M35.6378677,14.2571429 L29.8527462,4.3 C29.1062789,3.02857143 27.4697929,2.58571429 26.1778303,3.32857143 L16.1579425,9.07142857 C14.8659798,9.81428571 14.4353256,11.4428571 15.1674378,12.7142857 L20.9525593,22.6571429 C21.6990266,23.9285714 23.3355126,24.3714286 24.6274752,23.6285714 L34.647363,17.8857143 C35.9393256,17.1571429 36.384335,15.5285714 35.6378677,14.2571429 Z" opacity=".7"/>
			<path d="M28.0786542,9.01428571 L19.8962243,0.914285714 C18.8482991,-0.128571429 17.1400374,-0.128571429 16.0921121,0.914285714 L7.90968224,9.01428571 C6.86175701,10.0571429 6.86175701,11.7428571 7.90968224,12.7857143 L16.0921121,20.8857143 C17.1400374,21.9285714 18.8482991,21.9285714 19.8962243,20.8857143 L28.0786542,12.7857143 C29.1265794,11.7428571 29.1265794,10.0571429 28.0786542,9.01428571 Z" opacity=".5"/>
			<path d="M19.8531589,9.05714286 L9.83327103,3.32857143 C8.54130841,2.6 6.90482243,3.02857143 6.15835514,4.3 L0.373233645,14.2142857 C-0.373233645,15.4857143 0.0717757009,17.1142857 1.36373832,17.8571429 L11.3836262,23.5857143 C12.6755888,24.3142857 14.3120748,23.8857143 15.0585421,22.6142857 L20.8436636,12.7 C21.5757757,11.4142857 21.1451215,9.78571429 19.8531589,9.05714286 Z" opacity=".3"/>
			</g>
		</svg>
	},
	category: 'embed',
	attributes: {
		ridInput: {
			type: 'string'
		},
		ridId: {
			type: 'string'
		},
		error: {
			type: 'boolean'
		}
	},
	edit: ( props ) => {
		const {
			className,
			attributes: {
				ridInput,
				ridId,
				error
			},
			setAttributes,
		} = props;

		let inputTimeout = null

		// load embed.js via jsonp
		fetchJsonp('https://www.riddle.com/files/js/embed.js', {
			timeout: 2000,
		}).catch(function(ex) {
			// catch timeout error
			// and ignore :)
		})

		const checkRiddleInput = ( value ) => {
			// https://www.riddle.com/view/12345
			// https://www.riddle.com/showcase/12345/quiz
			// [rid=12345]
			// 12345

			let ridId = null

			if ( value.indexOf('https://www.riddle.com/view/') != -1) {
				ridId = value.split('https://www.riddle.com/view/')[1]
			} else if ( value.indexOf('https://www.riddle.com/showcase/') != -1) {
				ridId = value.split('https://www.riddle.com/showcase/')[1].split('/')[0]
			} else if ( value.indexOf('[rid=') != -1) {
				ridId = value.split('[rid=')[1].split(']')[0]
			}

			if ( isNaN(ridId) ) {
				ridId = null
			}

			if ( !isNaN(value) ) {
				ridId = value
			}

			if ( ridId != null) {
				return {
					status: 'ok',
					ridId: ridId,
					ridInput: value
				}
			}

			return {
				status: 'error',
				ridInput: value
			}
		}

		const handleInput = ( ridObject ) => {
			if ( ridObject.status == 'error' ) {
				// display error message
				setAttributes( { 
					ridId: ridObject.ridId,
					error: true
				} )
			} else {
				setAttributes( { 
					ridId: ridObject.ridId,
					error: false
				} )
			}
		}

		const onChangeRidInput = ( value ) => {
			setAttributes( { 
				ridInput: value
			} )

			clearTimeout(inputTimeout);

			inputTimeout = setTimeout(() => {
				let ridObject = checkRiddleInput( value )
				handleInput(ridObject)
			}, 1000)
		};

		return (
			<div className={ className }>
				<div class="wp-block-shortcode">
					<label>{__( 'URL, Shortcode or Riddle ID', 'riddle-plugin' )}</label>

					<PlainText
						placeholder={ __( 'URL, Shortcode or Riddle ID', 'riddle-plugin' ) }
						value={ ridInput }
						onChange={ onChangeRidInput }
					/>

					
				</div>
				{
					error && (
						<img src="https://www.riddle.com/assets/img/pommy.png" />
					)
				}
				{
					ridId && (
						<div className={ className }>
							<div class="riddle_target" data-rid-id={ridId} data-fg="#1486CD" data-bg="#FFFFFF" style={{ margin: '0 auto', maxWidth: '100%', width: '640px' }} data-auto-scroll="true" data-auto-scroll-offset="0">
								<link href="https://www.riddle.com/files/css/embed.css" rel="stylesheet"></link>
								<iframe style={{ width: '100%', height: '300px', border: '1px solid #cfcfcf' }} src={'https://www.riddle.com/a/' + ridId }></iframe>
							</div>
						</div>
					)
				}
			</div>
		);
	},
	save: ( props ) => {
		const {
			className,
			attributes: {
				ridId
			},
		} = props;
		return (
			<div className={ className }>
				[rid={ ridId }]
			</div>
		);
	},
} );