<?php

namespace src\classes\Landingpage\Service;

use Riddle\Landingpage\Module\Service\LeaderboardStoreService;

class LeaderboardSQLStoreService extends LeaderboardStoreService
{
    const LEADS_OPTION_NAME = 'riddle_leaderboard_leads';
    protected $calculatedMigration = false;

    protected function _loadLeaderboardLeadsFromSource(): array
    {
        $option = $this->_getOption();
        $leads = json_decode($option, true) ?? [];
        $this->_migrateOldLeads($leads);

        return $this->leads = $leads;
    }

    protected function _saveLeaderboardsFile()
    {
        if (file_exists($this->_getLeaderboardLeadsPath())) {
            \rename($this->_getLeaderboardLeadsPath(), $this->_getLeaderboardLeadsPath('_')); // rename the file so it does not get migrated again
        }

        if (null === $this->_getOption()) {
            \add_option(self::LEADS_OPTION_NAME, json_encode($this->leads));
        } else {
            \update_option(self::LEADS_OPTION_NAME, json_encode($this->leads));
        }
    }

    private function _getOption()
    {
        return \get_option(self::LEADS_OPTION_NAME, null);
    }

    /**
     * This function gets old leads and adds them to the news leads.
     */
    private function _migrateOldLeads(array &$leads)
    {
        $fileLeads = parent::_loadLeaderboardLeadsFromSource();
        
        if (empty($fileLeads)) {
            return;
        }

        $newKeyTable = $leads['keyTable'] ?? [];
        $entries = $fileLeads['entries'] ?? [];

        foreach ($entries as $entry) {
            $entryKey = $entry['key'];
            $keyIndex = $newKeyTable[$entryKey] ?? false;

            if (false === $keyIndex) { // not yet in the leads in general - just add the lead
                $leads['entries'][] = $entry;

                continue;
            }

            if (!$this->calculatedMigration) { // tedious work: add sums, add dates, ...
                $newEntry = $leads['entries'][$keyIndex];
                $newEntry['sumScoreNumber'] = $newEntry['sumScoreNumber'] + $entry['sumScoreNumber'];
                $newEntry['dates'] = array_merge($entry['dates'], $newEntry['dates']);
            } else { // already summed up values - it only gets loaded more than once
                $newEntry = $entry;
            }

            $leads['entries'][$keyIndex] = $newEntry;
        }

        $this->calculatedMigration = true;
        $this->refresh(); // sort leaderboard and refresh key table
    }
}