<?php

namespace src\controller;

use src\classes\Controller;
use src\classes\RiddleLoader;
use src\classes\UserSettings;
use src\classes\Landingpage\RiddleLandingpageManager;

class RiddleAdminController extends Controller
{   

    public static function riddleList() 
    {
        global $riddleLoader, $riddleApiKey;
        
        if (!$riddleApiKey) { // if the user hasn't put in a key yet
            return self::view("error-nokey-defined");
        }

        $team = self::_updateSelectedTeam($riddleLoader);
        $riddles = $riddleLoader->getRiddles();
        $teams = $riddleLoader->getTeamlist();

        if (!is_array($riddles)) {
            return self::view("error-invalidtoken");
        }

        return self::view("riddle-list", [
            'teams' => $teams,
            'riddles' => $riddles,
            'selectedTeam' => $team,
            'landingPages' => RiddleLandingpageManager::getLandingpagesRiddleIdArray(),
        ]);
    }

    public static function riddleUserSettings() 
    {
        $credentials = self::_getAPICredentials();
        
        return self::view("user-settings", $credentials);
    }

    public static function riddleProcessSettings() 
    {
        $key = self::_getPOSTValue('apikey');
        $bearer = self::_getPOSTValue('bearer');

        if ($key && $bearer) { // if the form has been submitted
            UserSettings::destroySettings();
            UserSettings::saveAPICredentials($key, $bearer);

            return self::redirectToAdminpage("riddle-admin-menu"); // Redirect to "My Riddles" page
        }
    }

    public static function riddleHelp() 
    {
        return self::view("riddle-help");
    }

    private static function _updateSelectedTeam(&$riddleLoader)
    {
        $teamId = self::_getPOSTValue('team', false);

        if ($teamId || $teamId === '') {
            UserSettings::setSelectedTeam($teamId);
        }

        $teamId = UserSettings::getSelectedTeam();

        if ($teamId) {
            $riddleLoader->setActiveTeam($teamId);

            return $teamId;
        }

        return false;
    }

    private static function _getAPICredentials()
    {
        return UserSettings::loadAPICredentials();
    }

}